@@ -6,7 +6,7 @@
 	# See the README and LICENSE files for details
 
 	# --------------------------------------------------------
-	# $Id: bug_update_advanced_page.php,v 1.83 2004/10/13 23:35:07 thraxisp Exp $
+	# $Id: bug_update_advanced_page.php,v 1.86 2005/06/12 21:04:43 thraxisp Exp $
 	# --------------------------------------------------------
 ?>
 <?php
@@ -24,6 +24,17 @@
 <?php
 	$f_bug_id = gpc_get_int( 'bug_id' );
 
+	$t_bug = bug_prepare_edit( bug_get( $f_bug_id, true ) );
+
+	if( $t_bug->project_id != helper_get_current_project() ) {
+		# in case the current project is not the same project of the bug we are viewing...
+		# ... override the current project. This to avoid problems with categories and handlers lists etc.
+		$g_project_override = $t_bug->project_id;
+		$t_changed_project = true;
+	} else {
+		$t_changed_project = false;
+	}
+
 	if ( SIMPLE_ONLY == config_get( 'show_update' ) ) {
 		print_header_redirect ( 'bug_update_page.php?bug_id=' . $f_bug_id );
 	}
@@ -35,7 +46,6 @@
 
 	access_ensure_bug_level( config_get( 'update_bug_threshold' ), $f_bug_id );
 
-	$t_bug = bug_prepare_edit( bug_get( $f_bug_id, true ) );
 ?>
 <?php html_page_top1( bug_format_summary( $f_bug_id, SUMMARY_CAPTION ) ) ?>
 <?php html_page_top2() ?>
@@ -92,6 +102,9 @@
 
 	<!-- Category -->
 	<td>
+		<?php if ( $t_changed_project ) {
+			echo "[" . project_get_field( $t_bug->project_id, 'name' ) . "] ";
+		} ?>
 		<select name="category">
 		<?php print_category_option_list( $t_bug->category, $t_bug->project_id ) ?>
 		</select>
@@ -228,8 +241,8 @@
 		<?php echo lang_get( 'status' ) ?>
 	</td>
 	<td bgcolor="<?php echo get_status_color( $t_bug->status ) ?>">
-		<?php print_status_option_list( 'status', $t_bug->status, 
-						( $t_bug->reporter_id == auth_get_current_user_id() && 
+		<?php print_status_option_list( 'status', $t_bug->status,
+						( $t_bug->reporter_id == auth_get_current_user_id() &&
 								( ON == config_get( 'allow_reporter_close' ) ) ), $t_bug->project_id ) ?>
 	</td>
 
@@ -301,17 +314,17 @@
 
 	<td class="category">
 		<?php
-			$t_show_version = ( ON == config_get( 'show_product_version' ) ) 
-					|| ( ( AUTO == config_get( 'show_product_version' ) ) 
+			$t_show_version = ( ON == config_get( 'show_product_version' ) )
+					|| ( ( AUTO == config_get( 'show_product_version' ) )
 								&& ( count( version_get_all_rows( $t_bug->project_id ) ) > 0 ) );
-			if ( $t_show_version ) { 
-				echo lang_get( 'fixed_in_version' ); 
+			if ( $t_show_version ) {
+				echo lang_get( 'fixed_in_version' );
 			}
 		?>
 	</td>
 	<td>
 		<?php
-			if ( $t_show_version ) { 
+			if ( $t_show_version ) {
 		?>
 		<select name="fixed_in_version">
 			<?php print_version_option_list( $t_bug->fixed_in_version, $t_bug->project_id, VERSION_ALL ) ?>
@@ -323,35 +336,35 @@
 
 	<!-- Product Version  or Product Build, if version is suppressed -->
 	<td class="category">
-		<?php 
-			if ( $t_show_version ) { 
+		<?php
+			if ( $t_show_version ) {
 				echo lang_get( 'product_version' );
-			}else{
+			} else {
 				echo lang_get( 'build' );
 			}
 		?>
 	</td>
 	<td>
-		<?php 
-			if ( $t_show_version ) { 
+		<?php
+			if ( $t_show_version ) {
 		?>
 		<select name="version">
 			<?php print_version_option_list( $t_bug->version, $t_bug->project_id, VERSION_RELEASED ) ?>
 		</select>
 		<?php
-			}else{
+			} else {
 		?>
 		<input type="text" name="build" size="16" maxlength="32" value="<?php echo $t_bug->build ?>" />
 		<?php
 			}
 		?>
-		
+
 	</td>
 
 </tr>
 
-<?php 
-	if ( $t_show_version ) { 
+<?php
+	if ( $t_show_version ) {
 ?>
 
 <tr <?php echo helper_alternate_class() ?>>
@@ -435,7 +448,7 @@
 	foreach( $t_related_custom_field_ids as $t_id ) {
 		$t_def = custom_field_get_definition( $t_id );
 		if( ( $t_def['display_update'] || $t_def['require_update']) && custom_field_has_write_access( $t_id, $f_bug_id ) ) {
-			$t_custom_fields_found = true;			
+			$t_custom_fields_found = true;
 ?>
 <tr <?php echo helper_alternate_class() ?>>
 	<td class="category">
