@@ -6,14 +6,13 @@
 	# See the README and LICENSE files for details
 
 	# --------------------------------------------------------
-	# $Id: string_api.php,v 1.62.4.1 2005/12/05 12:12:05 vboctor Exp $
+	# $Id: string_api.php,v 1.75.4.2.2.1.2.1.2.2 2006/07/05 03:04:41 thraxisp Exp $
 	# --------------------------------------------------------
 
 	$t_core_dir = dirname( __FILE__ ).DIRECTORY_SEPARATOR;
 
 	require_once( $t_core_dir . 'bug_api.php' );
 	require_once( $t_core_dir . 'user_pref_api.php' );
-	require_once( $t_core_dir . 'class.urlmatch.php' );
 
 	### String Processing API ###
 
@@ -60,6 +59,7 @@
 	# --------------------
 	# Similar to nl2br, but fixes up a problem where new lines are doubled between
 	# <pre> tags.
+	# additionally, wrap the text an $p_wrap character intervals if the config is set
 	function string_nl2br( $p_string, $p_wrap = 100 ) {
 		$p_string = nl2br( $p_string );
 
@@ -68,7 +68,12 @@
 		preg_match_all("/<pre[^>]*?>(.|\n)*?<\/pre>/", $p_string, $pre1);
 		for ( $x = 0; $x < count($pre1[0]); $x++ ) {
 			$pre2[$x] = preg_replace("/<br[^>]*?>/", "", $pre1[0][$x]);
-			$pre2[$x] = preg_replace("/([^\n]{".$p_wrap."})(?!<\/pre>)/", "$1\n", $pre2[$x]);
+			# @@@ thraxisp - this may want to be replaced by html_entity_decode (or equivalent)
+			#     if other encoded characters are a problem
+			$pre2[$x] = preg_replace("/&nbsp;/", " ", $pre2[$x]);
+			if ( ON == config_get( 'wrap_in_preformatted_text' ) ) {
+				$pre2[$x] = preg_replace("/([^\n]{".$p_wrap."})(?!<\/pre>)/", "$1\n", $pre2[$x]);
+			}
 			$pre1[0][$x] = "/" . preg_quote($pre1[0][$x], "/") . "/";
 		}
 
@@ -79,7 +84,7 @@
 	# Prepare a string for display to HTML
 	function string_display( $p_string ) {
 		$p_string = string_strip_hrefs( $p_string );
-		$p_string = htmlspecialchars( $p_string );
+		$p_string = string_html_specialchars( $p_string );
 		$p_string = string_restore_valid_html_tags( $p_string );
 		$p_string = string_preserve_spaces_at_bol( $p_string );
 		$p_string = string_nl2br( $p_string );
@@ -100,6 +105,28 @@
 		return $p_string;
 	}
 
+	# --------------------
+	# Prepare a string for display in rss
+	function string_rss_links( $p_string ) {
+		# rss can not start with &nbsp; which spaces will be replaced into by string_display().
+		$t_string = trim( $p_string );
+
+		# same steps as string_display_links() without the preservation of spaces since &nbsp; is undefined in XML.
+		$t_string = string_strip_hrefs( $t_string );
+		$t_string = string_html_specialchars( $t_string );
+		$t_string = string_restore_valid_html_tags( $t_string );
+		$t_string = string_nl2br( $t_string );
+		$t_string = string_insert_hrefs( $t_string );
+		$t_string = string_process_bug_link( $t_string, /* anchor */ true, /* detailInfo */ false, /* fqdn */ true );
+		$t_string = string_process_bugnote_link( $t_string, /* anchor */ true, /* detailInfo */ false, /* fqdn */ true );
+		$t_string = string_process_cvs_link( $t_string );
+
+		# another escaping to escape the special characters created by the generated links
+		$t_string = string_html_specialchars( $t_string );
+
+		return $t_string;
+	}
+
 	# --------------------
 	# Prepare a string for plain text display in email
 	function string_email( $p_string ) {
@@ -120,10 +147,12 @@
 		return $p_string;
 	}
 
+
+
 	# --------------------
 	# Process a string for display in a textarea box
 	function string_textarea( $p_string ) {
-		$p_string = htmlspecialchars( $p_string );
+		$p_string = string_html_specialchars( $p_string );
 
 		return $p_string;
 	}
@@ -131,7 +160,7 @@
 	# --------------------
 	# Process a string for display in a text box
 	function string_attribute( $p_string ) {
-		$p_string = htmlspecialchars( $p_string );
+		$p_string = string_html_specialchars( $p_string );
 
 		return $p_string;
 	}
@@ -144,6 +173,44 @@
 		return $p_string;
 	}
 
+	# --------------------
+	# validate the url as part of this site before continuing
+	function string_sanitize_url( $p_url ) {
+
+		$t_url = strip_tags( urldecode( $p_url ) );
+		if ( preg_match( '?http(s)*://?', $t_url ) > 0 ) { 
+			// no embedded addresses
+			if ( preg_match( '?^' . config_get( 'path' ) . '?', $t_url ) == 0 ) { 
+				// url is ok if it begins with our path, if not, replace it
+				$t_url = 'index.php';
+			}
+		}
+		if ( $t_url == '' ) {
+			$t_url = 'index.php';
+		}
+		
+		// split and encode parameters
+		if ( strpos( $t_url, '?' ) !== FALSE ) {
+			list( $t_path, $t_param ) = split( '\?', $t_url, 2 );
+			if ( $t_param !== "" ) {
+				$t_vals = array();
+				parse_str( $t_param, $t_vals );
+				$t_param = '';
+				foreach($t_vals as $k => $v) {
+					if ($t_param != '') {
+						$t_param .= '&'; 
+					}
+					$t_param .= "$k=" . urlencode( strip_tags( urldecode( $v ) ) );
+				}
+				return $t_path . '?' . $t_param;
+			} else {
+				return $t_path;
+			}
+		} else {
+			return $t_url;
+		}
+	}
+	
 	# --------------------
 	# process the $p_string and convert filenames in the format
 	#  cvs:filename.ext or cvs:filename.ext:n.nn to a html link
@@ -158,7 +225,7 @@
 			$t_replace_with = '[CVS] '.$t_cvs_web.'\\1?rev=\\4\\5';
 		}
 
-		return preg_replace( '/cvs:([^\.\s:,\?!]+(\.[^\.\s:,\?!]+)*)(:)?(\d\.[\d\.]+)?([\W\s])?/i',
+		return preg_replace( '/cvs:([^\.\s:,\?!<]+(\.[^\.\s:,\?!<]+)*)(:)?(\d\.[\d\.]+)?([\W\s])?/i',
 							 $t_replace_with,
 							 $p_string );
 	}
@@ -174,15 +241,15 @@
 	#
 	# The bug tag ('#' by default) must be at the beginning of the string or
 	#  preceeded by a character that is not a letter, a number or an underscore
-	function string_process_bug_link( $p_string, $p_include_anchor=true ) {
+	#
+	# if $p_include_anchor = false, $p_fqdn is ignored and assumed to true.
+	function string_process_bug_link( $p_string, $p_include_anchor = true, $p_detail_info = true, $p_fqdn = false ) {
 		$t_tag = config_get( 'bug_link_tag' );
 		# bail if the link tag is blank
 		if ( '' == $t_tag ) {
 			return $p_string;
 		}
 
-		$t_path = config_get( 'path' );
-
 		preg_match_all( '/(^|.+?)(?:(?<=^|\W)' . preg_quote($t_tag, '/') . '(\d+)|$)/s',
 								$p_string, $t_matches, PREG_SET_ORDER );
 		$t_result = '';
@@ -194,7 +261,7 @@
 				if ( isset( $t_match[2] ) ) {
 					$t_bug_id = $t_match[2];
 					if ( bug_exists( $t_bug_id ) ) {
-						$t_result .= string_get_bug_view_link( $t_bug_id, null );
+						$t_result .= string_get_bug_view_link( $t_bug_id, null, $p_detail_info, $p_fqdn );
 					} else {
 						$t_result .= $t_bug_id;
 					}
@@ -231,15 +298,15 @@
 	#
 	# The bugnote tag ('~' by default) must be at the beginning of the string or
 	#  preceeded by a character that is not a letter, a number or an underscore
-	function string_process_bugnote_link( $p_string, $p_include_anchor=true ) {
+	#
+	# if $p_include_anchor = false, $p_fqdn is ignored and assumed to true.
+	function string_process_bugnote_link( $p_string, $p_include_anchor = true, $p_detail_info = true, $p_fqdn = false ) {
 		$t_tag = config_get( 'bugnote_link_tag' );
 		# bail if the link tag is blank
 		if ( '' == $t_tag ) {
 			return $p_string;
 		}
 
-		$t_path = config_get( 'path' );
-
 		preg_match_all( '/(^|.+?)(?:(?<=^|\W)' . preg_quote($t_tag) . '(\d+)|$)/s',
 								$p_string, $t_matches, PREG_SET_ORDER );
 		$t_result = '';
@@ -253,7 +320,7 @@
 					if ( bugnote_exists( $t_bugnote_id ) ) {
 						$t_bug_id = bugnote_get_field( $t_bugnote_id, 'bug_id' );
 						if ( bug_exists( $t_bug_id ) ) {
-							$t_result .= string_get_bugnote_view_link( $t_bug_id, $t_bugnote_id, null );
+							$t_result .= string_get_bugnote_view_link( $t_bug_id, $t_bugnote_id, null, $p_detail_info, $p_fqdn );
 						} else {
 							$t_result .= $t_bugnote_id;
 						}
@@ -291,10 +358,20 @@
 		if ( !config_get( 'html_make_links' ) ) {
 			return $p_string;
 		}
+
+		$t_change_quotes = false;
+		if( ini_get_bool( 'magic_quotes_sybase' ) ) {
+			$t_change_quotes = true;
+			ini_set( 'magic_quotes_sybase', false );
+		}
+
 		# Find any URL in a string and replace it by a clickable link
-		$p_string = preg_replace( '/([http|irc|ftp|https]{2,}:\/\/([a-z0-9_-]|\/|\@|:{0,1}\.{0,1}){1,})/',
-									'<a href="\1">\1</a> [<a href="\1" target="blank">^</a>]',
-									$p_string);
+		$p_string = preg_replace( '/(([[:alpha:]][-+.[:alnum:]]*):\/\/(%[[:digit:]A-Fa-f]{2}|[-_.!~*\';\/?%^\\\\:@&={\|}+$#\(\),\[\][:alnum:]])+)/se',
+                                                                 "'<a href=\"'.rtrim('\\1','.').'\">\\1</a> [<a href=\"'.rtrim('\\1','.').'\" target=\"_blank\">^</a>]'",
+                                                                 $p_string);
+		if( $t_change_quotes ) {
+			ini_set( 'magic_quotes_sybase', true );
+		}
 
 		# Set up a simple subset of RFC 822 email address parsing
 		#  We don't allow domain literals or quoted strings
@@ -321,8 +398,9 @@
 		#  * whitespace
 		#  * a , character (allowing 'email foo@bar.baz, or ...')
 		#  * a \n, \r, or <
-		$p_string = preg_replace( '/(?<=^|&lt;|[\s\:\>\n\r])('.$t_atom.'(?:\.'.$t_atom.')*\@'.$t_atom.'(?:\.'.$t_atom.')*)(?=$|&gt;|[\s\,\<\n\r])/s',
-								'<a href="mailto:\1" target="_new">\1</a>',
+
+		$p_string = preg_replace( '/(?<=^|&quot;|&lt;|[\s\:\>\n\r])('.$t_atom.'(?:\.'.$t_atom.')*\@'.$t_atom.'(?:\.'.$t_atom.')*)(?=$|&quot;|&gt;|[\s\,\<\n\r])/s',
+								'<a href="mailto:\1">\1</a>',
 								$p_string);
 
 		return $p_string;
@@ -406,14 +484,16 @@
 	# --------------------
 	# return an href anchor that links to a bug VIEW page for the given bug
 	#  account for the user preference and site override
-	function string_get_bug_view_link( $p_bug_id, $p_user_id = null, $p_detail_info = true ) {
-		$t_link = "";
-
+	function string_get_bug_view_link( $p_bug_id, $p_user_id = null, $p_detail_info = true, $p_fqdn = false ) {
 		if ( bug_exists( $p_bug_id ) ) {
-			$t_summary	= string_attribute( bug_get_field( $p_bug_id, 'summary' ) );
-			$t_status	= string_attribute( get_enum_element( 'status', bug_get_field( $p_bug_id, 'status' ) ) );
-			$t_link		= '<a href="' . string_get_bug_view_url( $p_bug_id, $p_user_id ) . '"';
+			$t_link = '<a href="';
+			if ( $p_fqdn ) {
+				$t_link .= config_get( 'path' );
+			}
+			$t_link .= string_get_bug_view_url( $p_bug_id, $p_user_id ) . '"';
 			if ( $p_detail_info ) {
+				$t_summary = string_attribute( bug_get_field( $p_bug_id, 'summary' ) );
+				$t_status = string_attribute( get_enum_element( 'status', bug_get_field( $p_bug_id, 'status' ) ) );
 				$t_link .=  ' title="[' . $t_status . '] ' . $t_summary . '"';
 			}
 			$t_link .= '>' . bug_format_id( $p_bug_id ) . '</a>';
@@ -427,14 +507,17 @@
 	# --------------------
 	# return an href anchor that links to a bug VIEW page for the given bug
 	#  account for the user preference and site override
-	function string_get_bugnote_view_link( $p_bug_id, $p_bugnote_id, $p_user_id = null, $p_detail_info = true ) {
-		$t_link = "";
-
+	function string_get_bugnote_view_link( $p_bug_id, $p_bugnote_id, $p_user_id = null, $p_detail_info = true, $p_fqdn = false ) {
 		if ( bug_exists( $p_bug_id ) && bugnote_exists( $p_bugnote_id ) ) {
-			$t_reporter		= string_attribute( user_get_name ( bugnote_get_field( $p_bugnote_id, 'reporter_id' ) ) );
-			$t_update_date	= string_attribute( date( config_get( 'normal_date_format' ), ( db_unixtimestamp( bugnote_get_field( $p_bugnote_id, 'last_modified' ) ) ) ) );
-			$t_link		= '<a href="' . string_get_bugnote_view_url( $p_bug_id, $p_bugnote_id, $p_user_id ) . '"';
+			$t_link = '<a href="';
+			if ( $p_fqdn ) {
+				$t_link .= config_get( 'path' );
+			}
+
+			$t_link .= string_get_bugnote_view_url( $p_bug_id, $p_bugnote_id, $p_user_id ) . '"';
 			if ( $p_detail_info ) {
+				$t_reporter = string_attribute( user_get_name ( bugnote_get_field( $p_bugnote_id, 'reporter_id' ) ) );
+				$t_update_date = string_attribute( date( config_get( 'normal_date_format' ), ( db_unixtimestamp( bugnote_get_field( $p_bugnote_id, 'last_modified' ) ) ) ) );
 				$t_link .=  ' title="[' . $t_update_date . '] ' . $t_reporter . '"';
 			}
 			$t_link .= '>' . lang_get( 'bugnote' ) . ': ' . bugnote_format_id( $p_bugnote_id) . '</a>';
@@ -549,6 +632,74 @@
 		return date( config_get( 'complete_date_format' ), $t_timestamp );
 	}
 
+	# --------------------
+	# Shorten a string for display on a dropdown to prevent the page rendering too wide
+	#  ref issues #4630, #5072, #5131
+
+	function string_shorten( $p_string ) {
+		$t_max = config_get( 'max_dropdown_length' );
+		if ( ( strlen( $p_string ) > $t_max ) && ( $t_max > 0 ) ){
+			$t_pattern = '/([\s|.|,|\-|_|\/|\?]+)/';
+			$t_bits = preg_split( $t_pattern, $p_string, -1, PREG_SPLIT_DELIM_CAPTURE );
+
+			$t_string = '';
+			$t_last = $t_bits[ count( $t_bits ) - 1 ];
+			$t_last_len = strlen( $t_last );
+
+			foreach ( $t_bits as $t_bit ) {
+				if ( ( strlen( $t_string ) + strlen( $t_bit ) + $t_last_len + 3 <= $t_max )
+					|| ( strpos( $t_bit, '.,-/?' ) > 0 ) ) {
+					$t_string .= $t_bit;
+				} else {
+					break;
+				}
+			}
+			$t_string .= '...' . $t_last;
+			return $t_string;
+		} else {
+			return $p_string;
+		}
+	}
+
+	# --------------------
+	# remap a field name to a string name (for sort filter)
+
+	function string_get_field_name( $p_string ) {
+
+		$t_map = array(
+				'last_updated' => 'last_update',
+				'id' => 'email_bug'
+				);
+
+		$t_string = $p_string;
+		if ( isset( $t_map[ $p_string ] ) ) {
+			$t_string = $t_map[ $p_string ];
+		}
+		return lang_get_defaulted( $t_string );
+	}
+
+	# --------------------
+	# Calls htmlentities on the specified string, passing along
+	# the current charset, if the current PHP version supports it.
+	function string_html_entities( $p_string ) {
+		if ( php_version_at_least( '4.1.0' ) ) {
+			return htmlentities( $p_string, ENT_COMPAT, lang_get( 'charset' ) );
+		} else {
+			return htmlentities( $p_string );
+		}
+	}
+
+	# --------------------
+	# Calls htmlspecialchars on the specified string, passing along
+	# the current charset, if the current PHP version supports it.
+	function string_html_specialchars( $p_string ) {
+		if ( php_version_at_least( '4.1.0' ) ) {
+			return htmlspecialchars( $p_string, ENT_COMPAT, lang_get( 'charset' ) );
+		} else {
+			return htmlspecialchars( $p_string );
+		}
+	}
+	
 	# --------------------
 	# Prepares a string to be used as part of header().
 	function string_prepare_header( $p_string ) {
@@ -566,4 +717,4 @@
 
 		return $t_string;
 	}
-?>
\ No newline at end of file
+?>
