@@ -6,14 +6,14 @@
 	# See the README and LICENSE files for details
 
 	# --------------------------------------------------------
-	# $Id: print_bug_page.php,v 1.51 2004/08/27 00:29:54 thraxisp Exp $
+	# $Id: print_bug_page.php,v 1.56.6.1 2006/01/01 02:58:50 thraxisp Exp $
 	# --------------------------------------------------------
 ?>
 <?php
 	require_once( 'core.php' );
-	
+
 	$t_core_path = config_get( 'core_path' );
-	
+
 	require_once( $t_core_path.'bug_api.php' );
 	require_once( $t_core_path.'custom_field_api.php' );
 	require_once( $t_core_path.'date_api.php' );
@@ -30,15 +30,17 @@
 
 	$c_bug_id = (integer)$f_bug_id;
 
-	$query = "SELECT *, date_submitted, last_updated
-			FROM $g_mantis_bug_table
+	$t_bug_table = config_get( 'mantis_bug_table' );
+	$query = "SELECT *
+			FROM $t_bug_table
 			WHERE id='$c_bug_id'";
 	$result = db_query( $query );
 	$row = db_fetch_array( $result );
 	extract( $row, EXTR_PREFIX_ALL, 'v' );
 
+	$t_bug_text_table = config_get( 'mantis_bug_text_table' );
 	$query = "SELECT *
-			FROM $g_mantis_bug_text_table
+			FROM $t_bug_text_table
 			WHERE id='$v_bug_text_id'";
 	$result = db_query( $query );
 	$row = db_fetch_array( $result );
@@ -63,7 +65,7 @@
 <table class="width100" cellspacing="1">
 <tr>
 	<td class="form-title" colspan="6">
-		<div class="center"><?php echo config_get( 'window_title' ) . ' - ' . project_get_name( $v_project_id ) ?></div>
+		<div class="center"><?php echo config_get( 'window_title' ) . ' - ' . string_display( project_get_name( $v_project_id ) ) ?></div>
 	</td>
 </tr>
 <tr>
@@ -141,7 +143,11 @@
 		<?php echo lang_get( 'assigned_to' ) ?>:
 	</td>
 	<td class="print">
-		<?php print_user_with_subject( $v_handler_id, $f_bug_id ) ?>
+		<?php 
+			if ( access_has_bug_level( config_get( 'view_handler_threshold' ), $f_bug_id ) ) {
+				print_user_with_subject( $v_handler_id, $f_bug_id ); 
+			}
+		?>
 	</td>
 	<td class="print-category">
 		<?php echo lang_get( 'os' ) ?>:
@@ -292,8 +298,9 @@ foreach( $t_related_custom_field_ids as $t_id ) {
 <?php
 	# account profile description
 	if ( $v_profile_id > 0 ) {
+	    $t_user_prof_table = config_get( 'mantis_user_profile_table' );
 		$query = "SELECT description
-				FROM $g_mantis_user_profile_table
+				FROM $t_user_prof_table
 				WHERE id='$v_profile_id'";
 		$result = db_query( $query );
 		$t_profile_description = '';
@@ -327,30 +334,7 @@ foreach( $t_related_custom_field_ids as $t_id ) {
 		<?php echo lang_get( 'attached_files' ) ?>:
 	</td>
 	<td class="print" colspan="5">
-		<?php
-			$query = "SELECT filename, filesize, date_added
-					FROM $g_mantis_bug_file_table
-					WHERE bug_id='$c_bug_id'";
-			$result = db_query( $query );
-			$num_files = db_num_rows( $result );
-			for ($i=0;$i<$num_files;$i++) {
-				$row = db_fetch_array( $result );
-				extract( $row, EXTR_PREFIX_ALL, 'v2' );
-				$v2_filesize = round( $v2_filesize / 1024 );
-				$v2_date_added = date( config_get( 'normal_date_format' ), ( db_unixtimestamp( $v2_date_added ) ) );
-
-				switch ( $g_file_upload_method ) {
-					case DISK:	PRINT "$v2_filename ($v2_filesize KB) <span class=\"italic\">$v2_date_added</span>";
-							break;
-					case DATABASE:	PRINT "$v2_filename ($v2_filesize KB) <span class=\"italic\">$v2_date_added</span>";
-							break;
-				}
-
-				if ( $i != ($num_files - 1) ) {
-					PRINT '<br />';
-				}
-			}
-		?>
+		<?php file_list_attachments ( $f_bug_id ); ?>
 	</td>
 </tr>
 </table>
