@@ -6,18 +6,21 @@
 	# See the README and LICENSE files for details
 
 	# --------------------------------------------------------
-	# $Id: summary_graph_cumulative_bydate.php,v 1.12 2004/01/11 07:16:08 vboctor Exp $
+	# $Id: summary_graph_cumulative_bydate.php,v 1.14 2005/02/12 20:01:08 jlatour Exp $
 	# --------------------------------------------------------
 ?>
 <?php
 	require_once( 'core.php' );
-	
+
 	$t_core_path = config_get( 'core_path' );
-	
+
 	require_once( $t_core_path.'graph_api.php' );
 
 	access_ensure_project_level( config_get( 'view_summary_threshold' ) );
 
-	create_cumulative_bydate();
-	graph_cumulative_bydate();
-?>
\ No newline at end of file
+	$f_width = gpc_get_int( 'width', 300 );
+	$t_ar = config_get( 'graph_bar_aspect' );
+
+	$t_metrics = create_cumulative_bydate();
+	graph_cumulative_bydate( $t_metrics, $f_width, $f_width * $t_ar );
+?>
