@@ -8,9 +8,10 @@
 <?php
 	require_once( 'core.php' );
 	$t_core_path = config_get( 'core_path' );
-	
+
 	require_once( $t_core_path.'compress_api.php' );
 	require_once( $t_core_path.'filter_api.php' );
+	require_once( $t_core_path.'relationship_api.php' );
 	require_once( $t_core_path.'current_user_api.php' );
 	require_once( $t_core_path.'bug_api.php' );
 	require_once( $t_core_path.'string_api.php' );
@@ -23,42 +24,14 @@
 	html_page_top1();
 	html_page_top2();
 
-	$t_project_id = helper_get_current_project();
 	$t_target_field = gpc_get_string( 'target_field', '' );
-	$t_fields = array( 'priority', 'id', 'sponsorship_total', 'category', 'severity', 'status', 'last_updated', 'summary' );
-	$t_accessible_custom_fields_ids = array();
-	$t_accessible_custom_fields_names = array();
-	$t_accessible_custom_fields_values = array();
 
-	if ( ON == config_get( 'filter_by_custom_fields' ) ) {
-		$t_custom_cols = config_get( 'filter_custom_fields_per_row' );
-		$t_custom_fields = custom_field_get_ids( $t_project_id );
-
-		foreach ( $t_custom_fields as $t_cfid ) {
-			$t_field_info = custom_field_cache_row( $t_cfid, true );
-			if ( $t_field_info['access_level_r'] <= $t_current_user_access_level ) {
-				$t_accessible_custom_fields_ids[] = $t_cfid;
-				$t_accessible_custom_fields_names[] = $t_field_info['name'];
-				$t_accessible_custom_fields_values[] = custom_field_distinct_values( $t_cfid, $t_project_id );
-				$t_fields[] = "custom_" . $t_field_info['name'];
-			}
-		}
-
-		if ( sizeof( $t_accessible_custom_fields_ids ) > 0 ) {
-			$t_per_row = config_get( 'filter_custom_fields_per_row' );
-			$t_custom_rows = ceil( sizeof( $t_accessible_custom_fields_ids ) / $t_per_row );
-		}
-	}		
-
- 	if ( ! in_array( $t_target_field, $t_fields ) ) {
- 		$t_target_field = '';
- 	}
- 	
 	if ( ON == config_get( 'use_javascript' ) ) {
 		?>
 		<body onload="SetInitialFocus();">
-		
+
 		<script language="Javascript">
+		<!--
 		function SetInitialFocus() {
 			<?php
 			global $t_target_field;
@@ -76,10 +49,10 @@
 			if ( field_to_focus ) {
 				eval( "document.filters['" + field_to_focus + "'].focus()" );
 			}
-			
+
 			SwitchDateFields();
 		}
-		
+
 		function SwitchDateFields() {
 		    // All fields need to be enabled to go back to the script
 			document.filters.start_month.disabled = ! document.filters.do_filter_by_date.checked;
@@ -88,29 +61,66 @@
 			document.filters.end_month.disabled = ! document.filters.do_filter_by_date.checked;
 			document.filters.end_day.disabled = ! document.filters.do_filter_by_date.checked;
 			document.filters.end_year.disabled = ! document.filters.do_filter_by_date.checked;
-		
+
 		    return true;
 		}
+		// -->
 		</script>
 
 		<?php
 	}
-	
+
+	# @@@ thraxisp - could this be replaced by a call to filter_draw_selection_area2
+
 	$t_filter = current_user_get_bug_filter();
 	$t_filter = filter_ensure_valid_filter( $t_filter );
+	$t_project_id = helper_get_current_project();
 
-	$t_sort = $t_filter['sort'];
-	$t_dir = $t_filter['dir'];
-	
 	$t_current_user_access_level = current_user_get_access_level();
-	$t_filter_cols = 7;
+	$t_accessible_custom_fields_ids = array();
+	$t_accessible_custom_fields_names = array();
+	$t_accessible_custom_fields_type = array() ;
+	$t_accessible_custom_fields_values = array();
+	$t_filter_cols = config_get( 'filter_custom_fields_per_row' );
 	$t_custom_cols = 1;
 	$t_custom_rows = 0;
 
+	#get valid target fields
+	$t_fields = helper_call_custom_function( 'get_columns_to_view', array() );
+	$t_n_fields = count( $t_fields );
+	for ( $i=0; $i < $t_n_fields; $i++ ) {
+		if ( in_array( $t_fields[$i], array( 'selection', 'edit', 'bugnotes_count', 'attachment' ) ) ) {
+			unset( $t_fields[$i] );
+		}
+	}
+
+	if ( ON == config_get( 'filter_by_custom_fields' ) ) {
+		$t_custom_cols = $t_filter_cols;
+		$t_custom_fields = custom_field_get_linked_ids( $t_project_id );
+
+		foreach ( $t_custom_fields as $t_cfid ) {
+			$t_field_info = custom_field_cache_row( $t_cfid, true );
+			if ( $t_field_info['access_level_r'] <= $t_current_user_access_level ) {
+				$t_accessible_custom_fields_ids[] = $t_cfid;
+				$t_accessible_custom_fields_names[] = $t_field_info['name'];
+				$t_accessible_custom_fields_types[] = $t_field_info['type'];
+				$t_accessible_custom_fields_values[] = custom_field_distinct_values( $t_cfid, $t_project_id );
+				$t_fields[] = "custom_" . $t_field_info['name'];
+			}
+		}
+
+		if ( sizeof( $t_accessible_custom_fields_ids ) > 0 ) {
+			$t_per_row = config_get( 'filter_custom_fields_per_row' );
+			$t_custom_rows = ceil( sizeof( $t_accessible_custom_fields_ids ) / $t_per_row );
+		}
+	}
+
+	if ( ! in_array( $t_target_field, $t_fields ) ) {
+		$t_target_field = '';
+	}
+	
 	$f_for_screen = gpc_get_bool( 'for_screen', true );
 
-	$t_sort = $t_filter['sort'];
-	$t_dir = $t_filter['dir'];
 	$t_action  = "view_all_set.php?f=3";
 
 	if ( $f_for_screen == false ) {
@@ -137,22 +147,22 @@
 	if ( 'advanced' == $f_view_type ) {
 		$t_select_modifier = 'multiple="multiple" size="10" ';
 	}
-	
+
+	$t_show_version = ( ON == config_get( 'show_product_version' ) )
+			|| ( ( AUTO == config_get( 'show_product_version' ) )
+						&& ( count( version_get_all_rows_with_subs( $t_project_id ) ) > 0 ) );
 ?>
 <br />
 <form method="post" name="filters" action="<?php echo $t_action; ?>">
 <input type="hidden" name="type" value="1" />
 <input type="hidden" name="view_type" value="<?php PRINT $f_view_type; ?>" />
-<?php 
-	if ( $f_for_screen == false ) 
+<?php
+	if ( $f_for_screen == false )
 	{
 		print "<input type=\"hidden\" name=\"print\" value=\"1\" />";
 		print "<input type=\"hidden\" name=\"offset\" value=\"0\" />";
-	}	
+	}
 ?>
-<input type="hidden" name="sort" value="<?php echo $t_sort ?>" />
-<input type="hidden" name="dir" value="<?php echo $t_dir ?>" />
-
 <table class="width100" cellspacing="1">
 <tr>
 	<td class="right" colspan="<?php PRINT ( 8 * $t_custom_cols ); ?>">
@@ -176,86 +186,39 @@
 	<td class="small-caption" colspan="<?php echo ( 2 * $t_custom_cols ); ?>"><?php echo lang_get( 'category' ) ?></td>
 	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'severity' ) ?></td>
 	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'resolution' ) ?></td>
+	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'profile' ) ?></td>
+	<!-- <td colspan="<?php echo ( ( $t_filter_cols - 8 ) * $t_custom_cols ); ?>">&nbsp;</td> -->
 </tr>
 <tr class="row-1">
 	<!-- Reporter -->
 	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-		<select <?php PRINT $t_select_modifier;?> name="reporter_id[]">
-			<option value="any" <?php check_selected( $t_filter['reporter_id'], 'any' ); ?>><?php echo lang_get( 'any' ) ?></option>
-			<option value="any"></option>
-			<?php
-				if ( access_has_project_level( config_get( 'report_bug_threshold' ) ) ) { 
-					PRINT '<option value="' . META_FILTER_MYSELF . '" ';
-					check_selected( $t_filter['reporter_id'], META_FILTER_MYSELF );
-					PRINT '>[' . lang_get( 'myself' ) . ']</option>';
-				} 
-			?>
-			<?php print_reporter_option_list( $t_filter['reporter_id'] ) ?>
-		</select>
+		<?php print_filter_reporter_id(); ?>
 	</td>
 	<!-- Monitored by -->
 	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-		<select <?php PRINT $t_select_modifier;?> name="user_monitor[]">
-			<option value="any" <?php check_selected( $t_filter['user_monitor'], 'any' ); ?>><?php echo lang_get( 'any' ) ?></option>
-			<option value="any"></option>
-			<?php
-				if ( access_has_project_level( config_get( 'monitor_bug_threshold' ) ) ) {
-					PRINT '<option value="' . META_FILTER_MYSELF . '" ';
-					check_selected( $t_filter['user_monitor'], META_FILTER_MYSELF );
-					PRINT '>[' . lang_get( 'myself' ) . ']</option>';
-				}
-			?>
-			<?php print_reporter_option_list( $t_filter['user_monitor'] ) ?>
-		</select>
+		<?php print_filter_user_monitor(); ?>
 	</td>
 	<!-- Handler -->
 	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-		<select <?php PRINT $t_select_modifier;?> name="handler_id[]">
-			<option value="any" <?php check_selected( $t_filter['handler_id'], 'any' ); ?>><?php echo lang_get( 'any' ) ?></option>
-			<option value="none" <?php check_selected( $t_filter['handler_id'], 'none' ); ?>><?php echo lang_get( 'none' ) ?></option>
-			<option value="any"></option>
-			<?php
-				if ( access_has_project_level( config_get( 'handle_bug_threshold' ) ) ) { 
-					PRINT '<option value="' . META_FILTER_MYSELF . '" ';
-					check_selected( $t_filter['handler_id'], META_FILTER_MYSELF );
-					PRINT '>[' . lang_get( 'myself' ) . ']</option>';
-				} 
-			?>
-			<?php print_assign_to_option_list( $t_filter['handler_id'] ) ?>
-		</select>
-		<?php
-			if ( 'simple' == $f_view_type ) {
-			?>
-        		<br /><input type="checkbox" name="and_not_assigned" <?php check_checked( $t_filter['and_not_assigned'], 'on' ); ?> /> <?php echo lang_get( 'or_unassigned' ) ?>
-        	<?php
-        	}
-        ?>
+		<?php print_filter_handler_id(); ?>
 	</td>
 	<!-- Category -->
 	<td valign="top" colspan="<?php echo ( 2 * $t_custom_cols ); ?>">
-		<select <?php PRINT $t_select_modifier;?> name="show_category[]">
-			<option value="any" <?php check_selected( $t_filter['show_category'], 'any' ); ?>><?php echo lang_get( 'any' ) ?></option>
-			<option value="any"></option>
-			<?php # This shows orphaned categories as well as selectable categories ?>
-			<?php print_category_complete_option_list( $t_filter['show_category'] ) ?>
-		</select>
+		<?php print_filter_show_category(); ?>
 	</td>
     <!-- Severity -->
     <td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-        <select <?php PRINT $t_select_modifier;?> name="show_severity[]">
-			<option value="any" <?php check_selected( $t_filter['show_severity'], 'any' ); ?>><?php echo lang_get( 'any' ) ?></option>
-            <option value="any"></option>
-            <?php print_enum_string_option_list( 'severity', $t_filter['show_severity'] ) ?>
-        </select>
+		<?php print_filter_show_severity(); ?>
     </td>
 	<!-- Resolution -->
 	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-		<select <?php PRINT $t_select_modifier;?> name="show_resolution[]">
-			<option value="any" <?php check_selected( $t_filter['show_resolution'], 'any' ); ?>><?php echo lang_get( 'any' ) ?></option>
-			<option value="any"></option>
-			<?php print_enum_string_option_list( 'resolution', $t_filter['show_resolution'] ) ?>
-		</select>
+		<?php print_filter_show_resolution(); ?>
 	</td>
+	<!-- Profile -->
+	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
+		<?php print_filter_show_profile(); ?>
+	</td>
+	<!-- <td colspan="<?php echo ( ( $t_filter_cols - 8 ) * $t_custom_cols ); ?>">&nbsp;</td> -->
 </tr>
 
 <tr class="row-category2">
@@ -270,30 +233,26 @@
 	?>
 	</td>
 	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'product_build' ) ?></td>
-	<td class="small-caption" colspan="<?php echo ( 2 * $t_custom_cols ); ?>"><?php echo lang_get( 'product_version' ) ?></td>
-	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'fixed_in_version' ) ?></td>
+	<?php if ( $t_show_version ) { ?>
+		<td class="small-caption" colspan="<?php echo ( 2 * $t_custom_cols ); ?>"><?php echo lang_get( 'product_version' ) ?></td>
+		<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'fixed_in_version' ) ?></td>
+	<?php } else { ?>
+		<td class="small-caption" colspan="<?php echo ( 2 * $t_custom_cols ); ?>">&nbsp;</td>
+		<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">&nbsp;</td>
+	<?php } ?>
 	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'priority' ) ?></td>
+	<td class="small-caption" colspan="<?php echo ( ( $t_filter_cols - 7 ) * $t_custom_cols ); ?>">&nbsp;</td>
 </tr>
 <tr class="row-1">
 	<!-- Status -->
 	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-		<select <?php PRINT $t_select_modifier;?> name="show_status[]">
-			<option value="any" <?php check_selected( $t_filter['show_status'], 'any' ); ?>><?php echo lang_get( 'any' ) ?></option>
-			<option value="any"></option>
-			<?php print_enum_string_option_list( 'status', $t_filter['show_status'] ) ?>
-		</select>
+		<?php print_filter_show_status(); ?>
 	</td>
 	<!-- Hide Status -->
 	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
 	<?php
 	if ( 'simple' == $f_view_type ) {
-	?>
-		<select <?php PRINT $t_select_modifier;?> name="hide_status[]">
-			<option value="none"><?php echo lang_get( 'none' ) ?></option>
-			<option value="none"></option>
-			<?php print_enum_string_option_list( 'status', $t_filter['hide_status'] ) ?>
-		</select>
-	<?php
+		print_filter_hide_status();
 	} else {
 		echo '&nbsp;';
 	}
@@ -301,148 +260,84 @@
 	</td>
 	<!-- Build -->
 	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-		<select <?php PRINT $t_select_modifier;?> name="show_build[]">
-			<option value="any" <?php check_selected( $t_filter['show_build'], 'any' ); ?>><?php echo lang_get( 'any' ) ?></option>
-			<option value="any"></option>
-			<?php print_build_option_list( $t_filter['show_build'] ) ?>
-		</select>
+ 		<?php print_filter_show_build(); ?>
 	</td>
 	<!-- Version -->
 	<td valign="top" colspan="<?php echo ( 2 * $t_custom_cols ); ?>">
-		<select <?php PRINT $t_select_modifier;?> name="show_version[]">
-			<option value="any" <?php check_selected( $t_filter['show_version'], 'any' ); ?>><?php echo lang_get( 'any' ) ?></option>
-			<option value="any"></option>
-			<?php print_version_option_list( $t_filter['show_version'], null, VERSION_RELEASED ) ?>
-		</select>
+ 		<?php if ( $t_show_version ) {
+ 			print_filter_show_version();
+ 		} else {
+ 			echo "&nbsp;";
+ 		} ?>
 	</td>
 	<!-- Fixed in Version -->
 	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-		<select <?php PRINT $t_select_modifier;?> name="fixed_in_version[]">
-			<option value="any" <?php check_selected( $t_filter['fixed_in_version'], 'any' ); ?>><?php echo lang_get( 'any' ) ?></option>
-			<option value="any"></option>
-			<?php print_version_option_list( $t_filter['fixed_in_version'], null, VERSION_ALL ) ?>
-		</select>
-	</td>
+ 		<?php if ( $t_show_version ) {
+ 			print_filter_show_fixed_in_version();
+ 		} else {
+ 			echo "&nbsp;";
+ 		} ?>
+ 	</td>
 	<!-- Priority -->
-        <td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-                <select <?php PRINT $t_select_modifier;?> name="show_priority[]">
-                        <option value="any" <?php check_selected( $t_filter['show_priority'], 'any' ); ?>><?php echo lang_get( 'any' ) ?></option>
-                        <option value="any"></option>
-			<?php print_enum_string_option_list( 'priority', $t_filter['show_priority'] ) ?>
-                </select>
-        </td>
+	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
+		<?php print_filter_show_priority(); ?>
+	</td>
+	<td colspan="<?php echo ( ( $t_filter_cols - 7 ) * $t_custom_cols ); ?>">&nbsp;</td>
 </tr>
 
 <tr class="row-category2">
 	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'show' ) ?></td>
 	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'view_status' ) ?></td>
+	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'sticky' ) ?></td>
 	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'changed' ) ?></td>
-	<td class="small-caption" colspan="<?php echo ( 4 * $t_custom_cols ); ?>">
-		<input type="checkbox" name="do_filter_by_date" <?php 
-			check_checked( $t_filter['do_filter_by_date'], 'on' ); 
-			if ( ON == config_get( 'use_javascript' ) ) { 
+	<td class="small-caption" colspan="<?php echo ( 3 * $t_custom_cols ); ?>">
+		<input type="checkbox" name="do_filter_by_date" <?php
+			check_checked( $t_filter['do_filter_by_date'], 'on' );
+			if ( ON == config_get( 'use_javascript' ) ) {
 				print "onclick=\"SwitchDateFields();\""; } ?> />
 		<?php echo lang_get( 'use_date_filters' ) ?>
 	</td>
+	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
+		<?php echo lang_get( 'bug_relationships' ) ?>
+	</td>
+	<!-- <td colspan="<?php echo ( ( $t_filter_cols - 8 ) * $t_custom_cols ); ?>">&nbsp;</td> -->
 </tr>
 <tr class="row-2">
 	<!-- Number of bugs per page -->
 	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-		<input type="text" name="per_page" size="3" maxlength="7" value="<?php echo $t_filter['per_page'] ?>" />
+		<?php print_filter_per_page(); ?>
 	</td>
 	<!-- View Status -->
 	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-		<select name="view_state">
-			<?php
-			PRINT '<option value="any" ';
-			check_selected( $t_filter['view_state'], 'any' );
-			PRINT '>' . lang_get( 'any' ) . '</option>';
-			PRINT '<option value="any"></option>';
-			PRINT '<option value="' . VS_PUBLIC . '" ';
-			check_selected( $t_filter['view_state'], VS_PUBLIC );
-			PRINT '>' . lang_get( 'public' ) . '</option>';
-			PRINT '<option value="' . VS_PRIVATE . '" ';
-			check_selected( $t_filter['view_state'], VS_PRIVATE );
-			PRINT '>' . lang_get( 'private' ) . '</option>';
-			?>
-		</select>
+		<?php print_filter_view_state(); ?>
+	</td>
+	<!-- Show Sticky bugs -->
+	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
+		<?php print_filter_sticky_issues(); ?>
 	</td>
 	<!-- Highlight changed bugs -->
 	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-		<input type="text" name="highlight_changed" size="3" maxlength="7" value="<?php echo $t_filter['highlight_changed'] ?>" />
+		<?php print_filter_highlight_changed(); ?>
 	</td>
-	<td valign="top" class="left" colspan="<?php echo ( 4 * $t_custom_cols ); ?>">
-	<table cellspacing="0" cellpadding="0">
-	<!-- Start date -->
-	<tr>
-		<td>
-		<?php echo lang_get( 'start_date' ) ?>:
-		</td>
-		<td>
-		<?php
-		$t_chars = preg_split( '//', config_get( 'short_date_format' ), -1, PREG_SPLIT_NO_EMPTY );
-		foreach( $t_chars as $t_char ) {
-			if ( strcasecmp( $t_char, "M" ) == 0 ) {
-				print "<select name=\"start_month\">";
-				print_month_option_list( $t_filter['start_month'] );
-				print "</select>\n";
-			}
-			if ( strcasecmp( $t_char, "D" ) == 0 ) {
-				print "<select name=\"start_day\">";
-				print_day_option_list( $t_filter['start_day'] );
-				print "</select>\n";
-			}
-			if ( strcasecmp( $t_char, "Y" ) == 0 ) {
-				print "<select name=\"start_year\">";
-				print_year_option_list( $t_filter['start_year'] );
-				print "</select>\n";
-			}
-		}
-		?>
-		</td>
-	</tr>
-	<!-- End date -->
-	<tr>
-		<td>
-		<?php echo lang_get( 'end_date' ) ?>:
-		</td>
-		<td>
-		<?php
-		$t_chars = preg_split( '//', config_get( 'short_date_format' ), -1, PREG_SPLIT_NO_EMPTY );
-		foreach( $t_chars as $t_char ) {
-			if ( strcasecmp( $t_char, "M" ) == 0 ) {
-				print "<select name=\"end_month\">";
-				print_month_option_list( $t_filter['end_month'] );
-				print "</select>\n";
-			}
-			if ( strcasecmp( $t_char, "D" ) == 0 ) {
-				print "<select name=\"end_day\">";
-				print_day_option_list( $t_filter['end_day'] );
-				print "</select>\n";
-			}
-			if ( strcasecmp( $t_char, "Y" ) == 0 ) {
-				print "<select name=\"end_year\">";
-				print_year_option_list( $t_filter['end_year'] );
-				print "</select>\n";
-			}
-		}
-		?>
-		</td>
-	</tr>
-	</table>
+	<td valign="top" class="left" colspan="<?php echo ( 3 * $t_custom_cols ); ?>">
+		<?php print_filter_do_filter_by_date( true ); # hide checkbox as it's already been shown ?>
 	</td>
+	<td valign="top" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
+        <?php print_filter_relationship_type(); ?>
+	</td>
+	<!-- <td colspan="<?php echo ( ( $t_filter_cols - 8 ) * $t_custom_cols ); ?>">&nbsp;</td> -->
 </tr>
 
 <?php
 if ( ON == config_get( 'filter_by_custom_fields' ) ) {
 ?>
 	<?php # -- Custom Field Searching -- ?>
-	<?php 
+	<?php
 	if ( sizeof( $t_accessible_custom_fields_ids ) > 0 ) {
 		$t_per_row = config_get( 'filter_custom_fields_per_row' );
 		$t_num_rows = ceil( sizeof( $t_accessible_custom_fields_ids ) / $t_per_row );
 		$t_base = 0;
-		
+
 		for ( $i = 0; $i < $t_num_rows; $i++ ) {
 			?>
 			<tr class="row-category2">
@@ -458,32 +353,18 @@ if ( ON == config_get( 'filter_by_custom_fields' ) ) {
 			}
 			?>
 			</tr>
-			<tr>
+			<tr class="row-2">
 			<?php
 			for ( $j = 0; $j < $t_per_row; $j++ ) {
 				echo '<td colspan="' . ( 1 * $t_filter_cols ) . '">';
-				if ( isset( $t_accessible_custom_fields_names[$t_base + $j] ) ) {
-					echo '<select ' . $t_select_modifier . ' name="custom_field_' . $t_accessible_custom_fields_ids[$t_base + $j] .'[]">';
-					echo '<option value="any" ';
-					check_selected( $t_filter['custom_fields'][ $t_accessible_custom_fields_ids[$t_base + $j] ], 'any' );
-					echo '>' . lang_get( 'any' ) .'</option>';
-					echo '<option value=""></option>';
-					foreach( $t_accessible_custom_fields_values[$t_base + $j] as $t_item ) {
-						if ( ( strtolower( $t_item ) != "any" ) && ( trim( $t_item ) != "" ) ) {
-							echo '<option value="' .  htmlentities( $t_item )  . '" ';
-							if ( isset( $t_filter['custom_fields'][ $t_accessible_custom_fields_ids[$t_base + $j] ] ) ) {
-								check_selected( $t_filter['custom_fields'][ $t_accessible_custom_fields_ids[$t_base + $j] ], $t_item );
-							}
-							echo '>' . $t_item  . '</option>' . "\n";
-						}
-					}
-					echo '</select>';
+				if ( isset( $t_accessible_custom_fields_ids[$t_base + $j] ) ) {
+					print_filter_custom_field($t_accessible_custom_fields_ids[$t_base + $j]);
 				} else {
 					echo '&nbsp;';
 				}
 				echo '</td>';
 			}
-			
+
 			?>
 			</tr>
 			<?php
@@ -493,22 +374,33 @@ if ( ON == config_get( 'filter_by_custom_fields' ) ) {
 }
 ?>
 
+<tr class="row-1">
+	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>" valign="top">
+		<?php PRINT lang_get( 'sort' ) ?>:
+	</td>
+	<td valign="top" colspan="<?php echo ( ( $t_filter_cols - 1 ) * $t_custom_cols ); ?>">
+		<?php
+			print_filter_show_sort();
+		?>
+	</td>
+</tr>
 <tr class="row-category2">
 <td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"><?php echo lang_get( 'search' ) ?></td>
-<td class="small-caption" colspan="<?php echo ( 6 * $t_custom_cols ); ?>"></td>
+<td class="small-caption" colspan="<?php echo ( ( $t_filter_cols - 1 ) * $t_custom_cols ); ?>"></td>
 </tr>
 <tr>
 	<!-- Search field -->
 	<td colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
-		<input type="text" size="16" name="search" value="<?php echo htmlspecialchars( $t_filter['search'] ); ?>" />
+		<input type="text" size="16" name="search" value="<?php echo string_html_specialchars( $t_filter['search'] ); ?>" />
 	</td>
 
-	<td class="small-caption" colspan="<?php echo ( 5 * $t_custom_cols ); ?>"></td>
-			
+	<td class="small-caption" colspan="<?php echo ( ( $t_filter_cols - 3 ) * $t_custom_cols ); ?>"></td>
+
 	<!-- Submit button -->
 	<td class="right" colspan="<?php echo ( 1 * $t_custom_cols ); ?>">
 		<input type="submit" name="filter" class="button" value="<?php echo lang_get( 'filter_button' ) ?>" />
 	</td>
+	<td class="small-caption" colspan="<?php echo ( 1 * $t_custom_cols ); ?>"></td>
 </tr>
 </table>
 </form>
